unit CarRoundMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DirectDraw, DDUtil; //ddutil neni standard

type
  TCarForm = class(TForm)
    procedure FormActivate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    FPhase, FActive: Boolean;
    FAngle: Integer;
    FDD: IDirectDraw7;
    FDDSPrimary: IDirectDrawSurface7;
    FDDSBack: IDirectDrawSurface7;
    FDDSOne: IDirectDrawSurface7;
    FDDPal: IDirectDrawPalette;
    procedure FormSetCursor(var Msg: TMessage); message WM_SETCURSOR;
    procedure ErrorOut(hRet: hResult; FuncName: string);
    function  InitDirectDraw: Boolean;
    procedure FormOnIdle(Sender: TObject; var Done: Boolean);
    function  RestoreAll: hResult;
    function  FlipPages: Boolean;
    function  UpdateFrame: Boolean;
    procedure ExitDirectDraw;
  public
     { Public declarations }
  end;

var
  CarForm: TCarForm;

implementation

{$R *.DFM}

const szBitmap: PChar = 'all.bmp';

procedure TCarForm.ErrorOut(hRet: hResult; FuncName: string);
var s: string;
begin
  s := FuncName + ': ' + #13 + DDErrorString(hRet);
  MessageDlg(s, mtWarning, [mbOk], 0);
end;

procedure TCarForm.ExitDirectDraw;
begin
  if Assigned(FDD) then
  begin
    if Assigned(FDDSBack) then FDDSBack := nil;
    if Assigned(FDDSPrimary) then FDDSPrimary := nil;
    if Assigned(FDDSOne) then FDDSOne := nil;
    if Assigned(FDDPal) then FDDPal := nil;
    FDD := nil;
  end;
end;

function TCarForm.FlipPages: Boolean;
var hRet: hResult;
begin
  Result := False;

  // skusi vymenit surfacy
  while True do
  begin
    // vymen surfaces
    hRet := FDDSPrimary.Flip(nil, 0);
    if hRet = DD_OK then Break
      else if hRet = DDERR_SURFACELOST then // pokud se surf. ztratil -> obnovit
      begin
        hRet := RestoreAll;
        if hRet <> DD_OK then Exit;
      end else if hRet <> DDERR_WASSTILLDRAWING then Exit; // jina chyba
    end;

  // OK
  Result := True;
end;

procedure TCarForm.FormSetCursor(var Msg: TMessage);
begin
  SetCursor(0);
  Msg.Result := 1;
end;

function TCarForm.InitDirectDraw: Boolean;
var ddsd: TDDSurfaceDesc2;
    ddscaps: TDDSCaps2;
    hRet: hResult;
begin
  Result := False;

  // hlavni directdraw7
  hRet := DirectDrawCreateEx(nil, FDD, IDirectDraw7, nil);
  if hRet <> DD_OK then
  begin
    ErrorOut(hRet, 'DirectDrawCreateEx');
    Exit;
  end;

  // nastav kooperativu
  hRet := FDD.SetCooperativeLevel(Handle, DDSCL_FULLSCREEN or DDSCL_EXCLUSIVE or DDSCL_ALLOWREBOOT or DDSCL_ALLOWMODEX);
  if hRet <> DD_OK then
  begin
    ErrorOut(hRet, 'SetCooperativeLevel');
    Exit;
  end;

  // nastav graf. mod
  hRet := FDD.SetDisplayMode(320, 200, 8, 0, 0);
  if hRet <> DD_OK then
  begin
    ErrorOut(hRet, 'SetDisplayMode');
    Exit;
  end;

  // vytvori prim retezec surfacu s 1 backbufferem
  FillChar(ddsd, SizeOf(ddsd), 0);
  ddsd.dwSize := SizeOf(ddsd);
  ddsd.dwFlags := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
  ddsd.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or DDSCAPS_FLIP or DDSCAPS_COMPLEX;
  ddsd.dwBackBufferCount := 1;
  hRet := FDD.CreateSurface(ddsd, FDDSPrimary, nil);
  if hRet <> DD_OK then
  begin
    ErrorOut(hRet, 'CreateSurface');
    Exit;
  end;

  // vytvori pristup k backbufferu
  FillChar(ddscaps, SizeOf(ddscaps), 0);
  ddscaps.dwCaps := DDSCAPS_BACKBUFFER;
  hRet := FDDSPrimary.GetAttachedSurface(ddscaps, FDDSBack);
  if hRet <> DD_OK then
  begin
    ErrorOut(hRet, 'GetAttachedSurface');
    Exit;
  end;

  // vytvori a nastavi paletu
  FDDPal := DDLoadPalette(FDD, szBitmap);
  if FDDPal = nil then
  begin
    ErrorOut(hRet, 'DDLoadPalette');
    Exit;
  end;
  hRet := FDDSPrimary.SetPalette(FDDPal);
  if hRet <> DD_OK then
  begin
    ErrorOut(hRet, 'SetPalette');
    Exit;
  end;

  // vytvori offscreenove surface na nahrani nasi bitmapy
  FDDSOne := DDLoadBitmap(FDD, szBitmap, 0, 0);
  if FDDSOne = nil then
    begin
      ErrorOut(hRet, 'DDLoadBitmap');
      Exit;
    end;

  // nastavi cernou barvu jako pruhlednou
  DDSetColorKey(FDDSOne, RGB(0, 0, 0));

  // OK:
  Result := True;
end;

function TCarForm.RestoreAll: hResult;
var hRet: hResult;
begin
  hRet := FDDSPrimary._Restore;
  if hRet = DD_OK then
  begin
    hRet := FDDSOne._Restore;
    if hRet = DD_OK then hRet := DDReLoadBitmap(FDDSOne, szBitmap);
  end;
  Result := hRet;
end;

function TCarForm.UpdateFrame: Boolean;
var i, x, y, dx, dy: Integer;
    DestRect: TRect;
    hRet: hResult;
begin
  Result := False;

  // dame do backsurf. pozadi a ignorujeme neviditelnou barvu
  DestRect := Rect(0, 0, 319, 199);
  while True do
  begin
    hRet := FDDSBack.BltFast(0, 0, FDDSOne, @DestRect, DDBLTFAST_NOCOLORKEY);
    if hRet = DD_OK then Break;
    if hRet = DDERR_SURFACELOST then
    begin
      hRet := RestoreAll;
      if hRet <> DD_OK then Exit;
    end;
    if hRet <> DDERR_WASSTILLDRAWING then Exit;
  end;

  // vybereme auticko podle uhlu a dame ho na misto
  i := (FAngle mod 360) div 5;
  x := i mod 12;
  y := i div 12;
  DestRect := Rect(x * 20, 200 + y * 20, x * 20 + 19, 200 + y * 20 + 19);
  x := 149 + Round(-Cos(-FAngle * Pi / 180) * 70);
  y := 89 + Round(Sin(-FAngle * Pi / 180) * 70);
  while True do
  begin
    hRet := FDDSBack.BltFast(x, y, FDDSOne, @DestRect, DDBLTFAST_SRCCOLORKEY);
    if hRet = DD_OK then Break;
    if hRet = DDERR_SURFACELOST then
    begin
      hRet := RestoreAll;
      if hRet <> DD_OK then Exit;
    end;
    if hRet <> DDERR_WASSTILLDRAWING then Exit;
  end;

  // OK
  FAngle := FAngle + 1;
  if FAngle >= 360 then FAngle := 0;
  Result := True;
end;

procedure TCarForm.FormActivate(Sender: TObject);
begin
  FActive := True;
end;

procedure TCarForm.FormDeactivate(Sender: TObject);
begin
  FActive := False;
end;

procedure TCarForm.FormShow(Sender: TObject);
begin
  if not InitDirectDraw then Application.Terminate;
end;

procedure TCarForm.FormOnIdle(Sender: TObject; var Done: Boolean);
begin
  if FActive then
    if UpdateFrame then
    begin
      if not FlipPages then Close;
    end else Close;
  Done := False;
end;

procedure TCarForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ExitDirectDraw;
  Application.Minimize;
end;

procedure TCarForm.FormCreate(Sender: TObject);
begin
  SetWindowPos(Handle, HWND_TOPMOST, 0, 0, Screen.Width, Screen.Height, 0);
  FPhase := False; FAngle := 0;
  Application.OnIdle := FormOnIdle;
end;

procedure TCarForm.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_ESCAPE) or (Key = VK_F12) then Close;
end;

end.
